package domstorage

// Code generated by cdproto-gen. DO NOT EDIT.

// StorageID DOM Storage identifier.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/DOMStorage#type-StorageId
type StorageID struct {
	SecurityOrigin string `json:"securityOrigin"` // Security origin for the storage.
	IsLocalStorage bool   `json:"isLocalStorage"` // Whether the storage is local storage (not session storage).
}

// Item DOM Storage item.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/DOMStorage#type-Item
type Item []string
