package security

// Code generated by cdproto-gen. DO NOT EDIT.

// EventVisibleSecurityStateChanged the security state of the page changed.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/Security#event-visibleSecurityStateChanged
type EventVisibleSecurityStateChanged struct {
	VisibleSecurityState *VisibleSecurityState `json:"visibleSecurityState"` // Security state information about the page.
}

// EventSecurityStateChanged the security state of the page changed.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/Security#event-securityStateChanged
type EventSecurityStateChanged struct {
	SecurityState State               `json:"securityState"`     // Security state.
	Explanations  []*StateExplanation `json:"explanations"`      // List of explanations for the security state. If the overall security state is insecure or warning, at least one corresponding explanation should be included.
	Summary       string              `json:"summary,omitempty"` // Overrides user-visible description of the state.
}
