// Code generated by easyjson for marshaling/unmarshaling. DO NOT EDIT.

package audits

import (
	json "encoding/json"
	network "github.com/chromedp/cdproto/network"
	easyjson "github.com/mailru/easyjson"
	jlexer "github.com/mailru/easyjson/jlexer"
	jwriter "github.com/mailru/easyjson/jwriter"
)

// suppress unused package warning
var (
	_ *json.RawMessage
	_ *jlexer.Lexer
	_ *jwriter.Writer
	_ easyjson.Marshaler
)

func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits(in *jlexer.Lexer, out *SourceCodeLocation) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "url":
			out.URL = string(in.String())
		case "lineNumber":
			out.LineNumber = int64(in.Int64())
		case "columnNumber":
			out.ColumnNumber = int64(in.Int64())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits(out *jwriter.Writer, in SourceCodeLocation) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"url\":"
		out.RawString(prefix[1:])
		out.String(string(in.URL))
	}
	{
		const prefix string = ",\"lineNumber\":"
		out.RawString(prefix)
		out.Int64(int64(in.LineNumber))
	}
	{
		const prefix string = ",\"columnNumber\":"
		out.RawString(prefix)
		out.Int64(int64(in.ColumnNumber))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v SourceCodeLocation) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v SourceCodeLocation) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *SourceCodeLocation) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *SourceCodeLocation) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits1(in *jlexer.Lexer, out *SharedArrayBufferTransferIssueDetails) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "sourceCodeLocation":
			if in.IsNull() {
				in.Skip()
				out.SourceCodeLocation = nil
			} else {
				if out.SourceCodeLocation == nil {
					out.SourceCodeLocation = new(SourceCodeLocation)
				}
				(*out.SourceCodeLocation).UnmarshalEasyJSON(in)
			}
		case "isWarning":
			out.IsWarning = bool(in.Bool())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits1(out *jwriter.Writer, in SharedArrayBufferTransferIssueDetails) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"sourceCodeLocation\":"
		out.RawString(prefix[1:])
		if in.SourceCodeLocation == nil {
			out.RawString("null")
		} else {
			(*in.SourceCodeLocation).MarshalEasyJSON(out)
		}
	}
	{
		const prefix string = ",\"isWarning\":"
		out.RawString(prefix)
		out.Bool(bool(in.IsWarning))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v SharedArrayBufferTransferIssueDetails) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits1(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v SharedArrayBufferTransferIssueDetails) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits1(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *SharedArrayBufferTransferIssueDetails) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits1(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *SharedArrayBufferTransferIssueDetails) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits1(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits2(in *jlexer.Lexer, out *SameSiteCookieIssueDetails) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "cookie":
			if in.IsNull() {
				in.Skip()
				out.Cookie = nil
			} else {
				if out.Cookie == nil {
					out.Cookie = new(AffectedCookie)
				}
				(*out.Cookie).UnmarshalEasyJSON(in)
			}
		case "cookieWarningReasons":
			if in.IsNull() {
				in.Skip()
				out.CookieWarningReasons = nil
			} else {
				in.Delim('[')
				if out.CookieWarningReasons == nil {
					if !in.IsDelim(']') {
						out.CookieWarningReasons = make([]SameSiteCookieWarningReason, 0, 4)
					} else {
						out.CookieWarningReasons = []SameSiteCookieWarningReason{}
					}
				} else {
					out.CookieWarningReasons = (out.CookieWarningReasons)[:0]
				}
				for !in.IsDelim(']') {
					var v1 SameSiteCookieWarningReason
					(v1).UnmarshalEasyJSON(in)
					out.CookieWarningReasons = append(out.CookieWarningReasons, v1)
					in.WantComma()
				}
				in.Delim(']')
			}
		case "cookieExclusionReasons":
			if in.IsNull() {
				in.Skip()
				out.CookieExclusionReasons = nil
			} else {
				in.Delim('[')
				if out.CookieExclusionReasons == nil {
					if !in.IsDelim(']') {
						out.CookieExclusionReasons = make([]SameSiteCookieExclusionReason, 0, 4)
					} else {
						out.CookieExclusionReasons = []SameSiteCookieExclusionReason{}
					}
				} else {
					out.CookieExclusionReasons = (out.CookieExclusionReasons)[:0]
				}
				for !in.IsDelim(']') {
					var v2 SameSiteCookieExclusionReason
					(v2).UnmarshalEasyJSON(in)
					out.CookieExclusionReasons = append(out.CookieExclusionReasons, v2)
					in.WantComma()
				}
				in.Delim(']')
			}
		case "operation":
			(out.Operation).UnmarshalEasyJSON(in)
		case "siteForCookies":
			out.SiteForCookies = string(in.String())
		case "cookieUrl":
			out.CookieURL = string(in.String())
		case "request":
			if in.IsNull() {
				in.Skip()
				out.Request = nil
			} else {
				if out.Request == nil {
					out.Request = new(AffectedRequest)
				}
				(*out.Request).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits2(out *jwriter.Writer, in SameSiteCookieIssueDetails) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"cookie\":"
		out.RawString(prefix[1:])
		if in.Cookie == nil {
			out.RawString("null")
		} else {
			(*in.Cookie).MarshalEasyJSON(out)
		}
	}
	{
		const prefix string = ",\"cookieWarningReasons\":"
		out.RawString(prefix)
		if in.CookieWarningReasons == nil && (out.Flags&jwriter.NilSliceAsEmpty) == 0 {
			out.RawString("null")
		} else {
			out.RawByte('[')
			for v3, v4 := range in.CookieWarningReasons {
				if v3 > 0 {
					out.RawByte(',')
				}
				(v4).MarshalEasyJSON(out)
			}
			out.RawByte(']')
		}
	}
	{
		const prefix string = ",\"cookieExclusionReasons\":"
		out.RawString(prefix)
		if in.CookieExclusionReasons == nil && (out.Flags&jwriter.NilSliceAsEmpty) == 0 {
			out.RawString("null")
		} else {
			out.RawByte('[')
			for v5, v6 := range in.CookieExclusionReasons {
				if v5 > 0 {
					out.RawByte(',')
				}
				(v6).MarshalEasyJSON(out)
			}
			out.RawByte(']')
		}
	}
	{
		const prefix string = ",\"operation\":"
		out.RawString(prefix)
		(in.Operation).MarshalEasyJSON(out)
	}
	if in.SiteForCookies != "" {
		const prefix string = ",\"siteForCookies\":"
		out.RawString(prefix)
		out.String(string(in.SiteForCookies))
	}
	if in.CookieURL != "" {
		const prefix string = ",\"cookieUrl\":"
		out.RawString(prefix)
		out.String(string(in.CookieURL))
	}
	if in.Request != nil {
		const prefix string = ",\"request\":"
		out.RawString(prefix)
		(*in.Request).MarshalEasyJSON(out)
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v SameSiteCookieIssueDetails) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits2(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v SameSiteCookieIssueDetails) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits2(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *SameSiteCookieIssueDetails) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits2(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *SameSiteCookieIssueDetails) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits2(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits3(in *jlexer.Lexer, out *MixedContentIssueDetails) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "resourceType":
			(out.ResourceType).UnmarshalEasyJSON(in)
		case "resolutionStatus":
			(out.ResolutionStatus).UnmarshalEasyJSON(in)
		case "insecureURL":
			out.InsecureURL = string(in.String())
		case "mainResourceURL":
			out.MainResourceURL = string(in.String())
		case "request":
			if in.IsNull() {
				in.Skip()
				out.Request = nil
			} else {
				if out.Request == nil {
					out.Request = new(AffectedRequest)
				}
				(*out.Request).UnmarshalEasyJSON(in)
			}
		case "frame":
			if in.IsNull() {
				in.Skip()
				out.Frame = nil
			} else {
				if out.Frame == nil {
					out.Frame = new(AffectedFrame)
				}
				(*out.Frame).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits3(out *jwriter.Writer, in MixedContentIssueDetails) {
	out.RawByte('{')
	first := true
	_ = first
	if in.ResourceType != "" {
		const prefix string = ",\"resourceType\":"
		first = false
		out.RawString(prefix[1:])
		(in.ResourceType).MarshalEasyJSON(out)
	}
	{
		const prefix string = ",\"resolutionStatus\":"
		if first {
			first = false
			out.RawString(prefix[1:])
		} else {
			out.RawString(prefix)
		}
		(in.ResolutionStatus).MarshalEasyJSON(out)
	}
	{
		const prefix string = ",\"insecureURL\":"
		out.RawString(prefix)
		out.String(string(in.InsecureURL))
	}
	{
		const prefix string = ",\"mainResourceURL\":"
		out.RawString(prefix)
		out.String(string(in.MainResourceURL))
	}
	if in.Request != nil {
		const prefix string = ",\"request\":"
		out.RawString(prefix)
		(*in.Request).MarshalEasyJSON(out)
	}
	if in.Frame != nil {
		const prefix string = ",\"frame\":"
		out.RawString(prefix)
		(*in.Frame).MarshalEasyJSON(out)
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v MixedContentIssueDetails) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits3(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v MixedContentIssueDetails) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits3(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *MixedContentIssueDetails) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits3(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *MixedContentIssueDetails) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits3(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits4(in *jlexer.Lexer, out *InspectorIssueDetails) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "sameSiteCookieIssueDetails":
			if in.IsNull() {
				in.Skip()
				out.SameSiteCookieIssueDetails = nil
			} else {
				if out.SameSiteCookieIssueDetails == nil {
					out.SameSiteCookieIssueDetails = new(SameSiteCookieIssueDetails)
				}
				(*out.SameSiteCookieIssueDetails).UnmarshalEasyJSON(in)
			}
		case "mixedContentIssueDetails":
			if in.IsNull() {
				in.Skip()
				out.MixedContentIssueDetails = nil
			} else {
				if out.MixedContentIssueDetails == nil {
					out.MixedContentIssueDetails = new(MixedContentIssueDetails)
				}
				(*out.MixedContentIssueDetails).UnmarshalEasyJSON(in)
			}
		case "blockedByResponseIssueDetails":
			if in.IsNull() {
				in.Skip()
				out.BlockedByResponseIssueDetails = nil
			} else {
				if out.BlockedByResponseIssueDetails == nil {
					out.BlockedByResponseIssueDetails = new(BlockedByResponseIssueDetails)
				}
				(*out.BlockedByResponseIssueDetails).UnmarshalEasyJSON(in)
			}
		case "heavyAdIssueDetails":
			if in.IsNull() {
				in.Skip()
				out.HeavyAdIssueDetails = nil
			} else {
				if out.HeavyAdIssueDetails == nil {
					out.HeavyAdIssueDetails = new(HeavyAdIssueDetails)
				}
				(*out.HeavyAdIssueDetails).UnmarshalEasyJSON(in)
			}
		case "contentSecurityPolicyIssueDetails":
			if in.IsNull() {
				in.Skip()
				out.ContentSecurityPolicyIssueDetails = nil
			} else {
				if out.ContentSecurityPolicyIssueDetails == nil {
					out.ContentSecurityPolicyIssueDetails = new(ContentSecurityPolicyIssueDetails)
				}
				(*out.ContentSecurityPolicyIssueDetails).UnmarshalEasyJSON(in)
			}
		case "sharedArrayBufferTransferIssueDetails":
			if in.IsNull() {
				in.Skip()
				out.SharedArrayBufferTransferIssueDetails = nil
			} else {
				if out.SharedArrayBufferTransferIssueDetails == nil {
					out.SharedArrayBufferTransferIssueDetails = new(SharedArrayBufferTransferIssueDetails)
				}
				(*out.SharedArrayBufferTransferIssueDetails).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits4(out *jwriter.Writer, in InspectorIssueDetails) {
	out.RawByte('{')
	first := true
	_ = first
	if in.SameSiteCookieIssueDetails != nil {
		const prefix string = ",\"sameSiteCookieIssueDetails\":"
		first = false
		out.RawString(prefix[1:])
		(*in.SameSiteCookieIssueDetails).MarshalEasyJSON(out)
	}
	if in.MixedContentIssueDetails != nil {
		const prefix string = ",\"mixedContentIssueDetails\":"
		if first {
			first = false
			out.RawString(prefix[1:])
		} else {
			out.RawString(prefix)
		}
		(*in.MixedContentIssueDetails).MarshalEasyJSON(out)
	}
	if in.BlockedByResponseIssueDetails != nil {
		const prefix string = ",\"blockedByResponseIssueDetails\":"
		if first {
			first = false
			out.RawString(prefix[1:])
		} else {
			out.RawString(prefix)
		}
		(*in.BlockedByResponseIssueDetails).MarshalEasyJSON(out)
	}
	if in.HeavyAdIssueDetails != nil {
		const prefix string = ",\"heavyAdIssueDetails\":"
		if first {
			first = false
			out.RawString(prefix[1:])
		} else {
			out.RawString(prefix)
		}
		(*in.HeavyAdIssueDetails).MarshalEasyJSON(out)
	}
	if in.ContentSecurityPolicyIssueDetails != nil {
		const prefix string = ",\"contentSecurityPolicyIssueDetails\":"
		if first {
			first = false
			out.RawString(prefix[1:])
		} else {
			out.RawString(prefix)
		}
		(*in.ContentSecurityPolicyIssueDetails).MarshalEasyJSON(out)
	}
	if in.SharedArrayBufferTransferIssueDetails != nil {
		const prefix string = ",\"sharedArrayBufferTransferIssueDetails\":"
		if first {
			first = false
			out.RawString(prefix[1:])
		} else {
			out.RawString(prefix)
		}
		(*in.SharedArrayBufferTransferIssueDetails).MarshalEasyJSON(out)
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v InspectorIssueDetails) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits4(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v InspectorIssueDetails) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits4(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *InspectorIssueDetails) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits4(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *InspectorIssueDetails) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits4(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits5(in *jlexer.Lexer, out *InspectorIssue) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "code":
			(out.Code).UnmarshalEasyJSON(in)
		case "details":
			if in.IsNull() {
				in.Skip()
				out.Details = nil
			} else {
				if out.Details == nil {
					out.Details = new(InspectorIssueDetails)
				}
				(*out.Details).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits5(out *jwriter.Writer, in InspectorIssue) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"code\":"
		out.RawString(prefix[1:])
		(in.Code).MarshalEasyJSON(out)
	}
	{
		const prefix string = ",\"details\":"
		out.RawString(prefix)
		if in.Details == nil {
			out.RawString("null")
		} else {
			(*in.Details).MarshalEasyJSON(out)
		}
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v InspectorIssue) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits5(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v InspectorIssue) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits5(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *InspectorIssue) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits5(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *InspectorIssue) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits5(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits6(in *jlexer.Lexer, out *HeavyAdIssueDetails) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "resolution":
			(out.Resolution).UnmarshalEasyJSON(in)
		case "reason":
			(out.Reason).UnmarshalEasyJSON(in)
		case "frame":
			if in.IsNull() {
				in.Skip()
				out.Frame = nil
			} else {
				if out.Frame == nil {
					out.Frame = new(AffectedFrame)
				}
				(*out.Frame).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits6(out *jwriter.Writer, in HeavyAdIssueDetails) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"resolution\":"
		out.RawString(prefix[1:])
		(in.Resolution).MarshalEasyJSON(out)
	}
	{
		const prefix string = ",\"reason\":"
		out.RawString(prefix)
		(in.Reason).MarshalEasyJSON(out)
	}
	{
		const prefix string = ",\"frame\":"
		out.RawString(prefix)
		if in.Frame == nil {
			out.RawString("null")
		} else {
			(*in.Frame).MarshalEasyJSON(out)
		}
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v HeavyAdIssueDetails) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits6(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v HeavyAdIssueDetails) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits6(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *HeavyAdIssueDetails) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits6(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *HeavyAdIssueDetails) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits6(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits7(in *jlexer.Lexer, out *GetEncodedResponseReturns) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "body":
			out.Body = string(in.String())
		case "originalSize":
			out.OriginalSize = int64(in.Int64())
		case "encodedSize":
			out.EncodedSize = int64(in.Int64())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits7(out *jwriter.Writer, in GetEncodedResponseReturns) {
	out.RawByte('{')
	first := true
	_ = first
	if in.Body != "" {
		const prefix string = ",\"body\":"
		first = false
		out.RawString(prefix[1:])
		out.String(string(in.Body))
	}
	if in.OriginalSize != 0 {
		const prefix string = ",\"originalSize\":"
		if first {
			first = false
			out.RawString(prefix[1:])
		} else {
			out.RawString(prefix)
		}
		out.Int64(int64(in.OriginalSize))
	}
	if in.EncodedSize != 0 {
		const prefix string = ",\"encodedSize\":"
		if first {
			first = false
			out.RawString(prefix[1:])
		} else {
			out.RawString(prefix)
		}
		out.Int64(int64(in.EncodedSize))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v GetEncodedResponseReturns) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits7(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v GetEncodedResponseReturns) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits7(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *GetEncodedResponseReturns) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits7(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *GetEncodedResponseReturns) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits7(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits8(in *jlexer.Lexer, out *GetEncodedResponseParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "requestId":
			out.RequestID = network.RequestID(in.String())
		case "encoding":
			(out.Encoding).UnmarshalEasyJSON(in)
		case "quality":
			out.Quality = float64(in.Float64())
		case "sizeOnly":
			out.SizeOnly = bool(in.Bool())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits8(out *jwriter.Writer, in GetEncodedResponseParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"requestId\":"
		out.RawString(prefix[1:])
		out.String(string(in.RequestID))
	}
	{
		const prefix string = ",\"encoding\":"
		out.RawString(prefix)
		(in.Encoding).MarshalEasyJSON(out)
	}
	if in.Quality != 0 {
		const prefix string = ",\"quality\":"
		out.RawString(prefix)
		out.Float64(float64(in.Quality))
	}
	if in.SizeOnly {
		const prefix string = ",\"sizeOnly\":"
		out.RawString(prefix)
		out.Bool(bool(in.SizeOnly))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v GetEncodedResponseParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits8(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v GetEncodedResponseParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits8(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *GetEncodedResponseParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits8(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *GetEncodedResponseParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits8(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits9(in *jlexer.Lexer, out *EventIssueAdded) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "issue":
			if in.IsNull() {
				in.Skip()
				out.Issue = nil
			} else {
				if out.Issue == nil {
					out.Issue = new(InspectorIssue)
				}
				(*out.Issue).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits9(out *jwriter.Writer, in EventIssueAdded) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"issue\":"
		out.RawString(prefix[1:])
		if in.Issue == nil {
			out.RawString("null")
		} else {
			(*in.Issue).MarshalEasyJSON(out)
		}
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v EventIssueAdded) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits9(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v EventIssueAdded) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits9(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *EventIssueAdded) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits9(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *EventIssueAdded) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits9(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits10(in *jlexer.Lexer, out *EnableParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits10(out *jwriter.Writer, in EnableParams) {
	out.RawByte('{')
	first := true
	_ = first
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v EnableParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits10(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v EnableParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits10(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *EnableParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits10(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *EnableParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits10(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits11(in *jlexer.Lexer, out *DisableParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits11(out *jwriter.Writer, in DisableParams) {
	out.RawByte('{')
	first := true
	_ = first
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v DisableParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits11(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v DisableParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits11(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *DisableParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits11(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *DisableParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits11(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits12(in *jlexer.Lexer, out *ContentSecurityPolicyIssueDetails) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "blockedURL":
			out.BlockedURL = string(in.String())
		case "violatedDirective":
			out.ViolatedDirective = string(in.String())
		case "isReportOnly":
			out.IsReportOnly = bool(in.Bool())
		case "contentSecurityPolicyViolationType":
			(out.ContentSecurityPolicyViolationType).UnmarshalEasyJSON(in)
		case "frameAncestor":
			if in.IsNull() {
				in.Skip()
				out.FrameAncestor = nil
			} else {
				if out.FrameAncestor == nil {
					out.FrameAncestor = new(AffectedFrame)
				}
				(*out.FrameAncestor).UnmarshalEasyJSON(in)
			}
		case "sourceCodeLocation":
			if in.IsNull() {
				in.Skip()
				out.SourceCodeLocation = nil
			} else {
				if out.SourceCodeLocation == nil {
					out.SourceCodeLocation = new(SourceCodeLocation)
				}
				(*out.SourceCodeLocation).UnmarshalEasyJSON(in)
			}
		case "violatingNodeId":
			(out.ViolatingNodeID).UnmarshalEasyJSON(in)
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits12(out *jwriter.Writer, in ContentSecurityPolicyIssueDetails) {
	out.RawByte('{')
	first := true
	_ = first
	if in.BlockedURL != "" {
		const prefix string = ",\"blockedURL\":"
		first = false
		out.RawString(prefix[1:])
		out.String(string(in.BlockedURL))
	}
	{
		const prefix string = ",\"violatedDirective\":"
		if first {
			first = false
			out.RawString(prefix[1:])
		} else {
			out.RawString(prefix)
		}
		out.String(string(in.ViolatedDirective))
	}
	{
		const prefix string = ",\"isReportOnly\":"
		out.RawString(prefix)
		out.Bool(bool(in.IsReportOnly))
	}
	{
		const prefix string = ",\"contentSecurityPolicyViolationType\":"
		out.RawString(prefix)
		(in.ContentSecurityPolicyViolationType).MarshalEasyJSON(out)
	}
	if in.FrameAncestor != nil {
		const prefix string = ",\"frameAncestor\":"
		out.RawString(prefix)
		(*in.FrameAncestor).MarshalEasyJSON(out)
	}
	if in.SourceCodeLocation != nil {
		const prefix string = ",\"sourceCodeLocation\":"
		out.RawString(prefix)
		(*in.SourceCodeLocation).MarshalEasyJSON(out)
	}
	if in.ViolatingNodeID != 0 {
		const prefix string = ",\"violatingNodeId\":"
		out.RawString(prefix)
		out.Int64(int64(in.ViolatingNodeID))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v ContentSecurityPolicyIssueDetails) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits12(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v ContentSecurityPolicyIssueDetails) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits12(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *ContentSecurityPolicyIssueDetails) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits12(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *ContentSecurityPolicyIssueDetails) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits12(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits13(in *jlexer.Lexer, out *BlockedByResponseIssueDetails) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "request":
			if in.IsNull() {
				in.Skip()
				out.Request = nil
			} else {
				if out.Request == nil {
					out.Request = new(AffectedRequest)
				}
				(*out.Request).UnmarshalEasyJSON(in)
			}
		case "parentFrame":
			if in.IsNull() {
				in.Skip()
				out.ParentFrame = nil
			} else {
				if out.ParentFrame == nil {
					out.ParentFrame = new(AffectedFrame)
				}
				(*out.ParentFrame).UnmarshalEasyJSON(in)
			}
		case "blockedFrame":
			if in.IsNull() {
				in.Skip()
				out.BlockedFrame = nil
			} else {
				if out.BlockedFrame == nil {
					out.BlockedFrame = new(AffectedFrame)
				}
				(*out.BlockedFrame).UnmarshalEasyJSON(in)
			}
		case "reason":
			(out.Reason).UnmarshalEasyJSON(in)
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits13(out *jwriter.Writer, in BlockedByResponseIssueDetails) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"request\":"
		out.RawString(prefix[1:])
		if in.Request == nil {
			out.RawString("null")
		} else {
			(*in.Request).MarshalEasyJSON(out)
		}
	}
	if in.ParentFrame != nil {
		const prefix string = ",\"parentFrame\":"
		out.RawString(prefix)
		(*in.ParentFrame).MarshalEasyJSON(out)
	}
	if in.BlockedFrame != nil {
		const prefix string = ",\"blockedFrame\":"
		out.RawString(prefix)
		(*in.BlockedFrame).MarshalEasyJSON(out)
	}
	{
		const prefix string = ",\"reason\":"
		out.RawString(prefix)
		(in.Reason).MarshalEasyJSON(out)
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v BlockedByResponseIssueDetails) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits13(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v BlockedByResponseIssueDetails) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits13(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *BlockedByResponseIssueDetails) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits13(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *BlockedByResponseIssueDetails) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits13(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits14(in *jlexer.Lexer, out *AffectedRequest) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "requestId":
			out.RequestID = network.RequestID(in.String())
		case "url":
			out.URL = string(in.String())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits14(out *jwriter.Writer, in AffectedRequest) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"requestId\":"
		out.RawString(prefix[1:])
		out.String(string(in.RequestID))
	}
	if in.URL != "" {
		const prefix string = ",\"url\":"
		out.RawString(prefix)
		out.String(string(in.URL))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v AffectedRequest) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits14(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v AffectedRequest) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits14(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *AffectedRequest) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits14(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *AffectedRequest) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits14(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits15(in *jlexer.Lexer, out *AffectedFrame) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "frameId":
			(out.FrameID).UnmarshalEasyJSON(in)
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits15(out *jwriter.Writer, in AffectedFrame) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"frameId\":"
		out.RawString(prefix[1:])
		out.String(string(in.FrameID))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v AffectedFrame) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits15(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v AffectedFrame) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits15(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *AffectedFrame) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits15(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *AffectedFrame) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits15(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits16(in *jlexer.Lexer, out *AffectedCookie) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "name":
			out.Name = string(in.String())
		case "path":
			out.Path = string(in.String())
		case "domain":
			out.Domain = string(in.String())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits16(out *jwriter.Writer, in AffectedCookie) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"name\":"
		out.RawString(prefix[1:])
		out.String(string(in.Name))
	}
	{
		const prefix string = ",\"path\":"
		out.RawString(prefix)
		out.String(string(in.Path))
	}
	{
		const prefix string = ",\"domain\":"
		out.RawString(prefix)
		out.String(string(in.Domain))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v AffectedCookie) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits16(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v AffectedCookie) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoAudits16(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *AffectedCookie) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits16(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *AffectedCookie) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoAudits16(l, v)
}
